/* A GL window with a trackball interface. */

#include <stdio.h>
#include <stdlib.h>
#include "Fl_Gl_Tb_Window.h"

// Initialize the trackball window
void Fl_Gl_Tb_Window::init() {

	isIdleRedrawing = false;
	trackball.Reset();
  	mouse.SetWindowSize(w(), h());
}


/** Modification by Edward Vigmond to handle 1 button mice
    button         = FL_BUTTON1
   SHIFT + button = FL_BUTTON2
   CTRL  + button = FL_BUTTON3

   no combination with spin possible
*/
int Fl_Gl_Tb_Window::map_button( int e ) 
{
  int bmask=0;
  if( e&FL_BUTTON2 || ((e&B2_MIMIC_MASK)==B2_MIMIC_MASK )) bmask |= FL_BUTTON2;
  if( e&FL_BUTTON3 || ((e&B3_MIMIC_MASK)==B3_MIMIC_MASK )) bmask |= FL_BUTTON3;
  if( e&FL_BUTTON1 && (e&(FL_SHIFT|FL_CTRL))==0 ) bmask |= FL_BUTTON1;
  return bmask;
}


Fl_Gl_Tb_Window::~Fl_Gl_Tb_Window() {
}

#include <FL/names.h>
// The handle() method now does the actual work.
int Fl_Gl_Tb_Window::handle(int e) {
    static bool keymouse=false, keyredraw=false;
	switch (e) {
		case FL_PUSH:
			mouse.UpdatePos(Fl::event_x(),Fl::event_y());
			mouse.UpdatePos(Fl::event_x(),Fl::event_y());
			mouse.UpdateButtons(map_button(Fl::event_state()));
			break;
		case FL_RELEASE:
			mouse.UpdateButtons(map_button(Fl::event_state()));
			break;
		case FL_DRAG:
			mouse.UpdatePos(Fl::event_x(),Fl::event_y());
			mouse.UpdateButtons(map_button(Fl::event_state()));
			break;
        case FL_SHORTCUT:
            {
            int u=mouse.u, v=mouse.v;
            switch( Fl::event_key() ){
                case 'h' : //left
                    mouse.u -= 2;
                    break;
                case 'l' : //right
                    mouse.u += 2;
                    break;
                case 'j' : //left
                    mouse.v -= 2;
                    break;
                case 'k' : //right
                    mouse.v += 2;
                    break;
            }
            mouse.TransformScreenToOgl();
			mouse.UpdatePos( u, v );
			mouse.UpdateButtons(FL_BUTTON1);
            keymouse = true;
            }
            break;
        case FL_KEYUP:
            if( keymouse ) {
              keymouse = false;
              keyredraw = true;
            }
            break;
		default:
			// some other event, which might be resize
			mouse.SetWindowSize(w(), h());
			return 0;
	}
	trackball.Update();
	if (isIdleRedrawing || trackball.isSpinning || trackball.isChanged || keyredraw) {
		redraw();
        keyredraw = false;
		trackball.isChanged = false;
	}
	return 1;
}

